@@ -26,18 +26,18 @@ from fortls.constants import (
     log,
 )
 from fortls.ftypes import (
-    CLASS_info,
-    FUN_sig,
-    GEN_info,
-    INT_info,
+    ClassInfo,
+    FunSig,
+    GenProcDefInfo,
+    InterInfo,
     Range,
-    RESULT_sig,
-    SELECT_info,
-    SMOD_info,
-    SUB_info,
-    USE_info,
-    VAR_info,
-    VIS_info,
+    ResultSig,
+    SelectInfo,
+    SmodInfo,
+    SubInfo,
+    UseInfo,
+    VarInfo,
+    VisInfo,
 )
 from fortls.helper_functions import (
     detect_fixed_format,
@@ -51,24 +51,24 @@ from fortls.helper_functions import (
     strip_strings,
 )
 from fortls.objects import (
-    fortran_associate,
-    fortran_ast,
-    fortran_block,
-    fortran_do,
-    fortran_enum,
-    fortran_function,
-    fortran_if,
-    fortran_int,
-    fortran_meth,
-    fortran_module,
-    fortran_program,
-    fortran_scope,
-    fortran_select,
-    fortran_submodule,
-    fortran_subroutine,
-    fortran_type,
-    fortran_var,
-    fortran_where,
+    Associate,
+    Block,
+    Do,
+    Enum,
+    FortranAST,
+    Function,
+    If,
+    Interface,
+    Method,
+    Module,
+    Program,
+    Scope,
+    Select,
+    Submodule,
+    Subroutine,
+    Type,
+    Variable,
+    Where,
 )
 
 
@@ -169,47 +169,58 @@ def parse_var_keywords(test_str: str) -> tuple[list[str], str]:
 
 def read_var_def(line: str, var_type: str = None, fun_only: bool = False):
     """Attempt to read variable definition line"""
+
+    def parse_kind(line: str):
+        match = FRegex.KIND_SPEC.match(line)
+        if not match:
+            return None, line
+        kind_str = match.group(1).replace(" ", "")
+        line = line[match.end(0) :]
+        if kind_str.find("(") >= 0:
+            match_char = find_paren_match(line)
+            if match_char < 0:  # this triggers while typing with autocomplete
+                raise ValueError("Incomplete kind specification")
+            kind_str += line[: match_char + 1].strip()
+            line = line[match_char + 1 :]
+        return kind_str, line
+
     if var_type is None:
         type_match = FRegex.VAR.match(line)
         if type_match is None:
             return None
-        else:
-            var_type = type_match.group(0).strip()
-            trailing_line = line[type_match.end(0) :]
+        var_type = type_match.group(0).strip()
+        trailing_line = line[type_match.end(0) :]
     else:
         trailing_line = line[len(var_type) :]
     var_type = var_type.upper()
     trailing_line = trailing_line.split("!")[0]
     if len(trailing_line) == 0:
         return None
-    #
-    kind_match = FRegex.KIND_SPEC.match(trailing_line)
-    if kind_match:
-        kind_str = kind_match.group(1).replace(" ", "")
-        var_type += kind_str
-        trailing_line = trailing_line[kind_match.end(0) :]
-        if kind_str.find("(") >= 0:
-            match_char = find_paren_match(trailing_line)
-            if match_char < 0:
-                return None  # Incomplete type spec
-            else:
-                kind_word = trailing_line[: match_char + 1].strip()
-                var_type += kind_word
-                trailing_line = trailing_line[match_char + 1 :]
-    else:
-        # Class and Type statements need a kind spec
-        if var_type in ("TYPE", "CLASS"):
-            return None
-        # Make sure next character is space or comma or colon
-        if not trailing_line[0] in (" ", ",", ":"):
-            return None
+
+    # Parse the global kind, if any, for the current line definition
+    # The global kind in some cases, like characters can be overriden by a locally
+    # defined kind
+    try:
+        kind_str, trailing_line = parse_kind(trailing_line)
+        var_type += kind_str  # XXX: see below
+    except ValueError:
+        return None
+    except TypeError:  # XXX: remove with explicit kind specification in VarInfo
+        pass
+
+    # Class and Type statements need a kind spec
+    if not kind_str and var_type in ("TYPE", "CLASS"):
+        return None
+    # Make sure next character is space or comma or colon
+    if not kind_str and not trailing_line[0] in (" ", ",", ":"):
+        return None
     #
     keywords, trailing_line = parse_var_keywords(trailing_line)
     # Check if this is a function definition
-    fun_def = read_fun_def(trailing_line, RESULT_sig(type=var_type, keywords=keywords))
-    if (fun_def is not None) or fun_only:
+    fun_def = read_fun_def(trailing_line, ResultSig(type=var_type, keywords=keywords))
+    if fun_def or fun_only:
         return fun_def
-    #
+    # Split the type and variable name
     line_split = trailing_line.split("::")
     if len(line_split) == 1:
         if len(keywords) > 0:
@@ -222,8 +233,8 @@ def read_var_def(line: str, var_type: str = None, fun_only: bool = False):
         var_words = separate_def_list(trailing_line.strip())
         if var_words is None:
             var_words = []
-    #
-    return "var", VAR_info(var_type, keywords, var_words)
+
+    return "var", VarInfo(var_type, keywords, var_words, kind_str)
 
 
 def get_procedure_modifiers(
@@ -264,8 +275,8 @@ def get_procedure_modifiers(
 
 
 def read_fun_def(
-    line: str, result: RESULT_sig = None, mod_flag: bool = False
-) -> tuple[Literal["fun"], FUN_sig] | None:
+    line: str, result: ResultSig = None, mod_flag: bool = False
+) -> tuple[Literal["fun"], FunSig] | None:
     """Attempt to read FUNCTION definition line
 
     To infer the `result` `type` and `name` the variable definition is called
@@ -308,15 +319,15 @@ def read_fun_def(
     trailing_line = trailing_line.strip()
     results_match = FRegex.RESULT.match(trailing_line)
     if result is None:
-        result = RESULT_sig()
+        result = ResultSig()
     if results_match:
         result.name = results_match.group(1).strip().lower()
-    return "fun", FUN_sig(name, args, keywords, mod_flag, result)
+    return "fun", FunSig(name, args, keywords, mod_flag, result)
 
 
 def read_sub_def(
     line: str, mod_flag: bool = False
-) -> tuple[Literal["sub"], SUB_info] | None:
+) -> tuple[Literal["sub"], SubInfo] | None:
     """Attempt to read a SUBROUTINE definition line
 
     Parameters
@@ -339,7 +350,7 @@ def read_sub_def(
     if name is None:
         return None
 
-    return "sub", SUB_info(name, args, keywords, mod_flag)
+    return "sub", SubInfo(name, args, keywords, mod_flag)
 
 
 def read_block_def(line: str) -> tuple[Literal["block"], str] | None:
@@ -431,7 +442,7 @@ def read_select_def(line: str):
             if select_default_match is None:
                 return None
             else:
-                return "select", SELECT_info(4, None, None)
+                return "select", SelectInfo(4, None, None)
         select_type = 3
         select_desc = select_type_match.group(1).upper()
         select_binding = select_type_match.group(2)
@@ -443,7 +454,7 @@ def read_select_def(line: str):
         elif select_word.lower().startswith("type"):
             select_type = 2
         select_binding = select_match.group(2)
-    return "select", SELECT_info(select_type, select_binding, select_desc)
+    return "select", SelectInfo(select_type, select_binding, select_desc)
 
 
 def read_type_def(line: str):
@@ -485,7 +496,7 @@ def read_type_def(line: str):
     else:
         return None
     #
-    return "typ", CLASS_info(name, parent, keywords)
+    return "typ", ClassInfo(name, parent, keywords)
 
 
 def read_enum_def(line: str):
@@ -528,11 +539,16 @@ def read_generic_def(line: str):
     if len(pro_out) == 0:
         return None
     #
-    return "gen", GEN_info(bound_name, pro_out, vis_flag)
+    return "gen", GenProcDefInfo(bound_name, pro_out, vis_flag)
 
 
 def read_mod_def(line: str):
-    """Attempt to read MODULE and MODULE PROCEDURE definition lines"""
+    """Attempt to read MODULE and MODULE PROCEDURE, MODULE FUNCTION definition lines"""
+    # Get all the keyword modifier mathces
+    keywords = re.findall(FRegex.SUB_MOD, line)
+    # remove modifiers from line
+    line = re.sub(FRegex.SUB_MOD, "", line)
+
     mod_match = FRegex.MOD.match(line)
     if mod_match is None:
         return None
@@ -547,15 +563,19 @@ def read_mod_def(line: str):
         return "int_pro", pro_names
     # Check for submodule definition
     trailing_line = line[mod_match.start(1) :]
+    # module procedure
     sub_res = read_sub_def(trailing_line, mod_flag=True)
     if sub_res is not None:
         return sub_res
+    # module function
     fun_res = read_var_def(trailing_line, fun_only=True)
     if fun_res is not None:
         fun_res[1].mod_flag = True
-        return fun_res[0], fun_res[1]
+        fun_res[1].keywords = keywords
+        return fun_res
     fun_res = read_fun_def(trailing_line, mod_flag=True)
     if fun_res is not None:
+        fun_res[1].keywords = keywords
         return fun_res
     return "mod", name
 
@@ -581,7 +601,7 @@ def read_submod_def(line: str):
     name_match = FRegex.WORD.search(trailing_line)
     if name_match:
         name = name_match.group(0).lower()
-    return "smod", SMOD_info(name, parent_name)
+    return "smod", SmodInfo(name, parent_name)
 
 
 def read_prog_def(line: str) -> tuple[Literal["prog"], str] | None:
@@ -592,7 +612,7 @@ def read_prog_def(line: str) -> tuple[Literal["prog"], str] | None:
     return "prog", prog_match.group(1)
 
 
-def read_int_def(line: str) -> tuple[Literal["int"], INT_info] | None:
+def read_int_def(line: str) -> tuple[Literal["int"], InterInfo] | None:
     """Attempt to read INTERFACE definition line"""
     int_match = FRegex.INT.match(line)
     if int_match is None:
@@ -601,13 +621,13 @@ def read_int_def(line: str) -> tuple[Literal["int"], INT_info] | None:
     int_name = int_match.group(2).lower()
     is_abstract = int_match.group(1) is not None
     if int_name == "":
-        return "int", INT_info(None, is_abstract)
+        return "int", InterInfo(None, is_abstract)
     if int_name == "assignment" or int_name == "operator":
-        return "int", INT_info(None, False)
-    return "int", INT_info(int_match.group(2), is_abstract)
+        return "int", InterInfo(None, False)
+    return "int", InterInfo(int_match.group(2), is_abstract)
 
 
-def read_use_stmt(line: str) -> tuple[Literal["use"], USE_info] | None:
+def read_use_stmt(line: str) -> tuple[Literal["use"], UseInfo] | None:
     """Attempt to read USE statement"""
     use_match = FRegex.USE.match(line)
     if use_match is None:
@@ -624,7 +644,7 @@ def read_use_stmt(line: str) -> tuple[Literal["use"], USE_info] | None:
             only_list.add(only_name)
             if len(only_split) == 2:
                 rename_map[only_name] = only_split[1].strip()
-    return "use", USE_info(use_mod, only_list, rename_map)
+    return "use", UseInfo(use_mod, only_list, rename_map)
 
 
 def read_imp_stmt(line: str) -> tuple[Literal["import"], list[str]] | None:
@@ -648,7 +668,7 @@ def read_inc_stmt(line: str) -> tuple[Literal["inc"], str] | None:
     return "inc", inc_path
 
 
-def read_vis_stmnt(line: str) -> tuple[Literal["vis"], VIS_info] | None:
+def read_vis_stmnt(line: str) -> tuple[Literal["vis"], VisInfo] | None:
     """Attempt to read PUBLIC/PRIVATE statement"""
     vis_match = FRegex.VIS.match(line)
     if vis_match is None:
@@ -659,7 +679,7 @@ def read_vis_stmnt(line: str) -> tuple[Literal["vis"], VIS_info] | None:
         vis_type = 1
     trailing_line = line[vis_match.end(0) :].split("!")[0]
     mod_words = FRegex.WORD.findall(trailing_line)
-    return "vis", VIS_info(vis_type, mod_words)
+    return "vis", VisInfo(vis_type, mod_words)
 
 
 def_tests = [
@@ -686,9 +706,7 @@ def_tests = [
 ]
 
 
-def find_external_type(
-    file_ast: fortran_ast, desc_string: str, name_stripped: str
-) -> bool:
+def find_external_type(file_ast: FortranAST, desc_string: str, name: str) -> bool:
     """Encountered a variable with EXTERNAL as its type
     Try and find an already defined variable with a
     NORMAL Fortran Type"""
@@ -697,7 +715,7 @@ def find_external_type(
     counter = 0
     # Definition without EXTERNAL has already been parsed
     for v in file_ast.variable_list:
-        if name_stripped == v.name:
+        if name == v.name:
             # If variable is already in external objs it has
             # been parsed correctly so exit
             if v in file_ast.external_objs:
@@ -713,9 +731,7 @@ def find_external_type(
         return False
 
 
-def find_external_attr(
-    file_ast: fortran_ast, name_stripped: str, new_var: fortran_var
-) -> bool:
+def find_external_attr(file_ast: FortranAST, name: str, new_var: Variable) -> bool:
     """Check if this NORMAL Fortran variable is in the external_objs with only
     ``EXTERNAL`` as its type. Used to detect seperated ``EXTERNAL`` declarations.
 
@@ -723,7 +739,7 @@ def find_external_attr(
     ----------
     file_ast : fortran_ast
         AST file
-    name_stripped : str
+    name : str
         Variable name, stripped
     new_var : fortran_var
         Fortran variable to check against
@@ -736,7 +752,7 @@ def find_external_attr(
     """
     counter = 0
     for v in file_ast.external_objs:
-        if v.name != name_stripped:
+        if v.name != name:
             continue
         if v.desc.upper() != "EXTERNAL":
             continue
@@ -757,10 +773,10 @@ def find_external_attr(
 
 
 def find_external(
-    file_ast: fortran_ast,
+    file_ast: FortranAST,
     desc_string: str,
-    name_stripped: str,
-    new_var: fortran_var,
+    name: str,
+    new_var: Variable,
 ) -> bool:
     """Find a procedure, function, subroutine, etc. that has been defined as
     ``EXTERNAL``. ``EXTERNAL``s are parsed as ``fortran_var``, since there is no
@@ -788,7 +804,7 @@ def find_external(
         AST
     desc_string : str
         Variable type e.g. ``REAL``, ``INTEGER``, ``EXTERNAL``
-    name_stripped : str
+    name : str
         Variable name
     new_var : fortran_var
         The line variable that we are attempting to match with an ``EXTERNAL``
@@ -800,15 +816,15 @@ def find_external(
         True if the variable is ``EXTERNAL`` and we manage to link it to the
         rest of its components, else False
     """
-    if find_external_type(file_ast, desc_string, name_stripped):
+    if find_external_type(file_ast, desc_string, name):
         return True
     elif desc_string.upper() != "EXTERNAL":
-        if find_external_attr(file_ast, name_stripped, new_var):
+        if find_external_attr(file_ast, name, new_var):
             return True
     return False
 
 
-class fortran_file:
+class FortranFile:
     def __init__(self, path: str = None, pp_suffixes: list = None):
         self.path: str = path
         self.contents_split: list[str] = []
@@ -817,7 +833,7 @@ class fortran_file:
         self.nLines: int = 0
         self.fixed: bool = False
         self.preproc: bool = False
-        self.ast: fortran_ast = None
+        self.ast: FortranAST = None
         self.hash: str = None
         if path:
             _, file_ext = os.path.splitext(os.path.basename(path))
@@ -827,9 +843,9 @@ class fortran_file:
                 self.preproc = file_ext == file_ext.upper()
         self.COMMENT_LINE_MATCH, self.DOC_COMMENT_MATCH = self.get_comment_regexs()
 
-    def copy(self) -> fortran_file:
+    def copy(self) -> FortranFile:
         """Copy content to new file object (does not copy objects)"""
-        copy_obj = fortran_file(self.path)
+        copy_obj = FortranFile(self.path)
         copy_obj.preproc = self.preproc
         copy_obj.fixed = self.fixed
         copy_obj.contents_pp = self.contents_pp
@@ -1191,7 +1207,7 @@ class fortran_file:
         debug: bool = False,
         pp_defs: dict = None,
         include_dirs: set = None,
-    ) -> fortran_ast:
+    ) -> FortranAST:
         """Parse Fortran file contents of a fortran_file object and build an
         Abstract Syntax Tree (AST)
 
@@ -1221,7 +1237,7 @@ class fortran_file:
             )
 
         # This is not necessarily the same as self.ast
-        file_ast = fortran_ast(self)
+        file_ast = FortranAST(self)
         if self.preproc:
             log.debug("=== PreProc Pass ===\n")
             pp_skips, pp_defines = self.preprocess(
@@ -1335,76 +1351,76 @@ class fortran_file:
             if obj_type == "var":
                 if obj_info.var_names is None:
                     continue
-                desc_string = obj_info.var_type
                 link_name: str = None
                 procedure_def = False
-                if desc_string[:3] == "PRO":
+                if obj_info.var_type[:3] == "PRO":
                     if file_ast.current_scope.get_type() == INTERFACE_TYPE_ID:
                         for var_name in obj_info.var_names:
                             file_ast.add_int_member(var_name)
                         log.debug("%s !!! INTERFACE-PRO - Ln:%d", line.strip(), line_no)
                         continue
                     procedure_def = True
-                    link_name = get_paren_substring(desc_string)
+                    link_name = get_paren_substring(obj_info.var_type)
                 for var_name in obj_info.var_names:
+                    desc = obj_info.var_type
                     link_name: str = None
                     if var_name.find("=>") > -1:
                         name_split = var_name.split("=>")
-                        name_raw = name_split[0]
+                        name = name_split[0]
                         link_name = name_split[1].split("(")[0].strip()
                         if link_name.lower() == "null":
                             link_name = None
                     else:
-                        name_raw = var_name.split("=")[0]
+                        name = var_name.split("=")[0]
                     # Add dimension if specified
                     # TODO: turn into function and add support for co-arrays i.e. [*]
-                    key_tmp = obj_info.keywords[:]
-                    iparen = name_raw.find("(")
-                    if iparen == 0:
+                    # Copy global keywords to the individual variable
+                    var_keywords: list[str] = obj_info.keywords[:]
+                    # The name starts with (
+                    if name.find("(") == 0:
                         continue
-                    elif iparen > 0:
-                        if name_raw[iparen - 1] == "*":
-                            iparen -= 1
-                            if desc_string.find("(") < 0:
-                                desc_string += f"*({get_paren_substring(name_raw)})"
-                        else:
-                            key_tmp.append(
-                                f"dimension({get_paren_substring(name_raw)})"
-                            )
-                        name_raw = name_raw[:iparen]
-                    name_stripped = name_raw.strip()
-                    keywords, keyword_info = map_keywords(key_tmp)
+                    name, dims = self.parse_imp_dim(name)
+                    name, char_len = self.parse_imp_char(name)
+                    if dims:
+                        var_keywords.append(dims)
+                    if char_len:
+                        desc += char_len
+
+                    name = name.strip()
+                    keywords, keyword_info = map_keywords(var_keywords)
+
                     if procedure_def:
-                        new_var = fortran_meth(
+                        new_var = Method(
                             file_ast,
                             line_no,
-                            name_stripped,
-                            desc_string,
+                            name,
+                            desc,
                             keywords,
                             keyword_info=keyword_info,
                             link_obj=link_name,
                         )
                     else:
-                        new_var = fortran_var(
+                        new_var = Variable(
                             file_ast,
                             line_no,
-                            name_stripped,
-                            desc_string,
+                            name,
+                            desc,
                             keywords,
                             keyword_info=keyword_info,
+                            # kind=obj_info.var_kind,
                             link_obj=link_name,
                         )
                         # If the object is fortran_var and a parameter include
                         #  the value in hover
                         if new_var.is_parameter():
-                            _, col = find_word_in_line(line, name_stripped)
+                            _, col = find_word_in_line(line, name)
                             match = FRegex.PARAMETER_VAL.match(line[col:])
                             if match:
                                 var = match.group(1).strip()
                                 new_var.set_parameter_val(var)
 
                         # Check if the "variable" is external and if so cycle
-                        if find_external(file_ast, desc_string, name_stripped, new_var):
+                        if find_external(file_ast, desc, name, new_var):
                             continue
 
                     # if not merge_external:
@@ -1412,25 +1428,25 @@ class fortran_file:
                 log.debug("%s !!! VARIABLE - Ln:%d", line, line_no)
 
             elif obj_type == "mod":
-                new_mod = fortran_module(file_ast, line_no, obj_info)
+                new_mod = Module(file_ast, line_no, obj_info)
                 file_ast.add_scope(new_mod, FRegex.END_MOD)
                 log.debug("%s !!! MODULE - Ln:%d", line, line_no)
 
             elif obj_type == "smod":
-                new_smod = fortran_submodule(
+                new_smod = Submodule(
                     file_ast, line_no, obj_info.name, ancestor_name=obj_info.parent
                 )
                 file_ast.add_scope(new_smod, FRegex.END_SMOD)
                 log.debug("%s !!! SUBMODULE - Ln:%d", line, line_no)
 
             elif obj_type == "prog":
-                new_prog = fortran_program(file_ast, line_no, obj_info)
+                new_prog = Program(file_ast, line_no, obj_info)
                 file_ast.add_scope(new_prog, FRegex.END_PROG)
                 log.debug("%s !!! PROGRAM - Ln:%d", line, line_no)
 
             elif obj_type == "sub":
                 keywords, _ = map_keywords(obj_info.keywords)
-                new_sub = fortran_subroutine(
+                new_sub = Subroutine(
                     file_ast,
                     line_no,
                     obj_info.name,
@@ -1443,7 +1459,7 @@ class fortran_file:
 
             elif obj_type == "fun":
                 keywords, _ = map_keywords(obj_info.keywords)
-                new_fun = fortran_function(
+                new_fun = Function(
                     file_ast,
                     line_no,
                     obj_info.name,
@@ -1458,7 +1474,7 @@ class fortran_file:
                 # result() variable that is the function name
                 if obj_info.result.type:
                     keywords, keyword_info = map_keywords(obj_info.result.keywords)
-                    new_obj = fortran_var(
+                    new_obj = Variable(
                         file_ast,
                         line_no,
                         name=obj_info.result.name,
@@ -1474,7 +1490,7 @@ class fortran_file:
                 if name is None:
                     counters["block"] += 1
                     name = f"#BLOCK{counters['block']}"
-                new_block = fortran_block(file_ast, line_no, name)
+                new_block = Block(file_ast, line_no, name)
                 file_ast.add_scope(new_block, FRegex.END_BLOCK, req_container=True)
                 log.debug("%s !!! BLOCK - Ln:%d", line, line_no)
 
@@ -1483,7 +1499,7 @@ class fortran_file:
                 name = f"#DO{counters['do']}"
                 if obj_info != "":
                     block_id_stack.append(obj_info)
-                new_do = fortran_do(file_ast, line_no, name)
+                new_do = Do(file_ast, line_no, name)
                 file_ast.add_scope(new_do, FRegex.END_DO, req_container=True)
                 log.debug("%s !!! DO - Ln:%d", line, line_no)
 
@@ -1492,14 +1508,14 @@ class fortran_file:
                 if not obj_info:
                     counters["do"] += 1
                     name = f"#WHERE{counters['do']}"
-                    new_do = fortran_where(file_ast, line_no, name)
+                    new_do = Where(file_ast, line_no, name)
                     file_ast.add_scope(new_do, FRegex.END_WHERE, req_container=True)
                 log.debug("%s !!! WHERE - Ln:%d", line, line_no)
 
             elif obj_type == "assoc":
                 counters["block"] += 1
                 name = f"#ASSOC{counters['block']}"
-                new_assoc = fortran_associate(file_ast, line_no, name)
+                new_assoc = Associate(file_ast, line_no, name)
                 file_ast.add_scope(new_assoc, FRegex.END_ASSOCIATE, req_container=True)
                 for bound_var in obj_info:
                     try:
@@ -1519,14 +1535,14 @@ class fortran_file:
             elif obj_type == "if":
                 counters["if"] += 1
                 name = f"#IF{counters['if']}"
-                new_if = fortran_if(file_ast, line_no, name)
+                new_if = If(file_ast, line_no, name)
                 file_ast.add_scope(new_if, FRegex.END_IF, req_container=True)
                 log.debug("%s !!! IF - Ln:%d", line, line_no)
 
             elif obj_type == "select":
                 counters["select"] += 1
                 name = f"#SELECT{counters['select']}"
-                new_select = fortran_select(file_ast, line_no, name, obj_info)
+                new_select = Select(file_ast, line_no, name, obj_info)
                 file_ast.add_scope(new_select, FRegex.END_SELECT, req_container=True)
                 new_var = new_select.create_binding_variable(
                     file_ast,
@@ -1540,7 +1556,7 @@ class fortran_file:
 
             elif obj_type == "typ":
                 keywords, _ = map_keywords(obj_info.keywords)
-                new_type = fortran_type(file_ast, line_no, obj_info.name, keywords)
+                new_type = Type(file_ast, line_no, obj_info.name, keywords)
                 if obj_info.parent is not None:
                     new_type.set_inherit(obj_info.parent)
                 file_ast.add_scope(new_type, FRegex.END_TYPED, req_container=True)
@@ -1549,7 +1565,7 @@ class fortran_file:
             elif obj_type == "enum":
                 counters["block"] += 1
                 name = f"#ENUM{counters['block']}"
-                new_enum = fortran_enum(file_ast, line_no, name)
+                new_enum = Enum(file_ast, line_no, name)
                 file_ast.add_scope(new_enum, FRegex.END_ENUMD, req_container=True)
                 log.debug("%s !!! ENUM - Ln:%d", line, line_no)
 
@@ -1558,14 +1574,12 @@ class fortran_file:
                 if name is None:
                     counters["interface"] += 1
                     name = f"#GEN_INT{counters['interface']}"
-                new_int = fortran_int(
-                    file_ast, line_no, name, abstract=obj_info.abstract
-                )
+                new_int = Interface(file_ast, line_no, name, abstract=obj_info.abstract)
                 file_ast.add_scope(new_int, FRegex.END_INT, req_container=True)
                 log.debug("%s !!! INTERFACE - Ln:%d", line, line_no)
 
             elif obj_type == "gen":
-                new_int = fortran_int(
+                new_int = Interface(
                     file_ast, line_no, obj_info.bound_name, abstract=False
                 )
                 new_int.set_visibility(obj_info.vis_flag)
@@ -1583,7 +1597,7 @@ class fortran_file:
                         log.debug("%s !!! INTERFACE-PRO - Ln:%d", line, line_no)
 
                     elif file_ast.current_scope.get_type() == SUBMODULE_TYPE_ID:
-                        new_impl = fortran_scope(file_ast, line_no, obj_info[0])
+                        new_impl = Scope(file_ast, line_no, obj_info[0])
                         file_ast.add_scope(new_impl, FRegex.END_PRO)
                         log.debug("%s !!! INTERFACE-IMPL - Ln:%d", line, line_no)
 
@@ -1636,8 +1650,58 @@ class fortran_file:
                     log.debug(f"{error['range']}: {error['message']}")
         return file_ast
 
+    def parse_imp_dim(self, line: str):
+        """Parse the implicit dimension of an array e.g.
+        var(3,4), var_name(size(val,1)*10)
+
+        Parameters
+        ----------
+        line : str
+            line containing variable name
+
+        Returns
+        -------
+        tuple[str, str]
+            truncated line, dimension string
+        """
+        m = re.compile(r"[ ]*\w+[ ]*(\()", re.I).match(line)
+        if not m:
+            return line, None
+        i = find_paren_match(line[m.end(1) :])
+        if i < 0:
+            return line, None  # triggers for autocomplete
+        dims = line[m.start(1) : m.end(1) + i + 1]
+        line = line[: m.start(1)] + line[m.end(1) + i + 1 :]
+        return line, f"dimension{dims}"
+
+    def parse_imp_char(self, line: str):
+        """Parse the implicit character length from a variable e.g.
+        var_name*10 or var_name*(10), var_name*(size(val, 1))
+
+        Parameters
+        ----------
+        line : str
+            line containing potential variable
+
+        Returns
+        -------
+        tuple[str, str]
+            truncated line, character length
+        """
+        match = re.compile(r"(\w+)[ ]*\*[ ]*(\d+|\()", re.I).match(line)
+        if not match:
+            return line, None
+        if match.group(2) == "(":
+            i = find_paren_match(line[match.end(2) :])
+            if i < 0:
+                return line, None  # triggers for autocomplete
+            char_len = line[match.start(2) : match.end(2) + i + 1]
+        elif match.group(2).isdigit():
+            char_len = match.group(2)
+        return match.group(1), f"*{char_len}"
+
     def parse_end_scope_word(
-        self, line: str, ln: int, file_ast: fortran_ast, match: re.Match
+        self, line: str, ln: int, file_ast: FortranAST, match: re.Match
     ) -> bool:
         """Parses END keyword marking the end of scopes
 
@@ -1685,7 +1749,7 @@ class fortran_file:
         self,
         line: str,
         ln: int,
-        file_ast: fortran_ast,
+        file_ast: FortranAST,
         line_label: str,
         block_id_stack: list[str],
     ):
@@ -1703,7 +1767,7 @@ class fortran_file:
                 return True
         return False
 
-    def parse_implicit(self, line: str, ln: int, file_ast: fortran_ast) -> bool:
+    def parse_implicit(self, line: str, ln: int, file_ast: FortranAST) -> bool:
         """Parse implicit statements from a line
 
         Parameters
@@ -1735,7 +1799,7 @@ class fortran_file:
         log.debug("%s !!! IMPLICIT - Ln:%d", line, ln)
         return True
 
-    def parse_contains(self, line: str, ln: int, file_ast: fortran_ast) -> bool:
+    def parse_contains(self, line: str, ln: int, file_ast: FortranAST) -> bool:
         """Parse contain statements
 
         Parameters
@@ -1768,7 +1832,7 @@ class fortran_file:
         log.debug("%s !!! CONTAINS - Ln:%d", line, ln)
         return True
 
-    def parse_docs(self, line: str, ln: int, file_ast: fortran_ast, docs: list[str]):
+    def parse_docs(self, line: str, ln: int, file_ast: FortranAST, docs: list[str]):
         """Parse documentation stings of style Doxygen or FORD.
         Multiline docstrings are detected if the first comment starts with `!>`
         docstring continuations are detected with either `!>`, `!<` or `!!`
@@ -1790,7 +1854,7 @@ class fortran_file:
                 return f"!! {docs[0]}"
             return "!! " + "\n!! ".join(docs)
 
-        def add_line_comment(file_ast: fortran_ast, docs: list[str]):
+        def add_line_comment(file_ast: FortranAST, docs: list[str]):
             # Handle dangling comments from previous line
             if docs:
                 file_ast.add_doc(format(docs))
@@ -2083,7 +2147,7 @@ def preprocess_file(
                     break
             if include_path is not None:
                 try:
-                    include_file = fortran_file(include_path)
+                    include_file = FortranFile(include_path)
                     err_string, _ = include_file.load_from_disk()
                     if err_string is None:
                         log.debug(f'\n!!! Parsing include file "{include_path}"')
